;*************************************************
;*       YM2413 player for FX-870P/VX-4/3        *
;*     (C)2010 Miyura (miyura321@ybb.ne.jp)      *
;*************************************************

FX870P:	EQU	1		;FX-870P/VX-4
VX3:	EQU	2		;VX-3

;-------------------------------------------------
MODEL:	EQU	FX870P
;-------------------------------------------------

#if MODEL=FX870P
CSRDT:	EQU	&H1135		;cursor brink buffer
#else
CSRDT:	EQU	&H623C		;cursor brink buffer
#endif

	ORG	CSRDT + 24
	START	INIT
;
;	draw keyboard graphic
;
INIT:	PRE	IX,CGDAT
	PRE	IZ,ORDER
	XRW	$9,$9
	LD	$8,&H82		;left side
	CAL	DRWKB		;
	LD	$8,&H92		;right side
	CAL	DRWKB		;
;
;	reset YM2413B
;
	PST	PE,&HE0
	PST	PD,$31		;reset port5
;
;	initialize resistors
;
	LDW	$20,&H0230	;$20: VOL=-0dB, INST=3(piano)
				;$21: tone data offset = 2
	PST	PD,&HE0		;set port5
;
;	set original tone
;
	CAL	SETIO		;set I/O address (bank3)
	PRE	IZ,ORGDT
	LD	$0,$31
SET01:	STD	$0,(IX+$31)
	LDI	$1,(IZ+$31)
	ST	$1,(IX+$30)
	AD	$0,$30
	SBC	$0,8
	JR	NZ,SET01
	PST	UA,&H55		;set bank1
;
;	scan control keys
;
KYSCN:	PRE	IX,KYDTM
	LD	$0,7		;$0=counter
KYS01:	SB	$0,$30
	JR	C,SCNON

	LDIW	$11,(IX+$31)	;$11=IA, $12=KY
	PST	IA,$11
	LDCM	$0,$0,8		;wait
	GRE	KY,$3
	ANC	$4,$12		;check key code
	JR	Z,KYS01

	SBC	$0,$30
	JR	C,INCVL		;$0=0:[+]
	JR	Z,DECVL		;$0=1:[-]
	SBC	$0,3
	JR	C,INCKY		;$0=2:[/]
	JR	Z,DECKY		;$0=3:[*]
	SBC	$0,5
	JR	C,INCIN		;$0=4:[CLS]
	JR	Z,DECIN		;$0=5:[^]

QUIT:	PST	UA,&H54		;$0=6:[BS]
	RTN
;
;	change instrument
;
INCIN:	AD	$20,&H10,KYHLD	;increment instrument No.
DECIN:	SB	$20,&H10,KYHLD	;decrement instrument No.
;
;	change key base
;
INCKY:	SBC	$21,4		;
	JR	Z,KYHLD		;increment key
	AD	$21,$30,KYHLD	;
DECKY:	SBC	$21,$31		;
	JR	Z,KYHLD		;decrement key
	SB	$21,$30,KYHLD	;
;
;	volume control
;
INCVL:	SBC	$20,$31		;
	JR	LZ,KYHLD	;increment volume
	SB	$20,$30,KYHLD	;
DECVL:	ADC	$20,$30		;
	JR	LZ,KYHLD	;decrement volume
	AD	$20,$30		;

KYHLD:	GRE	KY,$3		;
	ANC	$4,$12		;check key hold
	JR	NZ,KYHLD	;
	JR	KYSCN
;
;	scan newly pressed key
;
SCNON:	LDIW	$11,(IX+$31)	;$11=IA, $12=KY
	PST	IA,$11
	AD	$0,$30
	SBC	$0,60
	JR	Z,SCOFF
	GRE	KY,$3
	ANC	$3,$12		;check key code
	JR	Z,SCNON		;

	SBC	$0,11		;$0=11:[BRK]
	JR	Z,QUIT		;
;
;	skip already pressed keys
;
	PRE	IZ,KYBUF
	LD	$2,9
KYS02:	LDI	$1,(IZ+$31)
	SBC	$1,$0
	JR	Z,SCNON
	SB	$2,$30
	JR	NZ,KYS02
;
;	select channel
;
	PRE	IX,KYBUF
	LD	$22,9
KYS03:	SB	$22,$30		;all channels are used -> C=1
	JR	C,SCOFF		;
	ADC	(IX+$22),$30	;skip used channel
	JR	NC,KYS03	;
	STD	$0,(IX+$22)	;write code of newly pressed key

	CAL	DRAW		;draw finger graphic
;
;	make a sound
;
	CAL	CLFPR		;calc F parameter -> $11,12
	CAL	SETIO		;set I/O address (bank3)

	LD	$0,&H30
	OR	$0,$22
	STD	$0,(IX+$31)	;set INST,VOL
	ST	$20,(IX+$30)	;
	LD	$0,&H10
	OR	$0,$22
	STD	$0,(IX+$31)	;set F parameter
	ST	$11,(IX+$30)	;
	LD	$0,&H20
	OR	$0,$22
	STD	$0,(IX+$31)	;sound on
	ST	$12,(IX+$30)	;
	PST	UA,&H55		;set bank1
;
;	scan released key
;
SCOFF:	PRE	IX,KYBUF
	PRE	IZ,KYDAT
	LD	$22,9
KYS04:	SB	$22,$30
	JP	C,KYSCN

	LD	$0,(IX+$22)
	ADC	$0,$30
	JR	C,KYS04

	LD	$5,$0
	BIU	$0
	LDW	$11,(IZ+$0)
	PST	IA,$11
	LDCM	$0,$0,8		;wait
	GRE	KY,$3
	ANC	$3,$12
	JR	NZ,KYS04
;
;	sound off
;
	NA	$1,$31		;write code FFh 
	STD	$1,(IX+$22)	;
	LD	$0,$5
	CAL	CLFPR		;calc F parameter -> $11,12
	CAL	SETIO		;set I/O address (bank3)
	LD	$0,&H20
	OR	$0,$22
	STD	$0,(IX+$31)
	AN	$12,&HEF
	ST	$12,(IX+$30)
	PST	UA,&H55		;set bank1

	LD	$0,$5
	CAL	ERASE		;erase finger
	JP	KYSCN
;
;	draw finger
;
DRAW:	LDW	$15,&HB07F	;
	LDW	$17,&HB0B0	;set finger graphic
	LD	$19,&H7F,DRW00
;
;	erase finger
;
ERASE:	ANC	$0,$30
	JR	NZ,ERA01
	PRE	IZ,WHITE
	JR	ERA02
ERA01:	PRE	IZ,BLACK
ERA02:	LDM	$15,(IZ+$31),5	;load key graphic data

DRW00:	LD	$5,$0
	SBC	$0,29		;check G#2
	JR	Z,GS2DR		;
	JR	C,LCD1		;select LCD1/LCD2 ($8)
	SB	$5,30
	LDW	$8,&H0092	;LCD2
	JR	YSET
LCD1:	LDW	$8,&H0C82	;LCD1
;
;	set Y address
;
YSET:	LD	$6,$5
	BID	$6
	JR	C,YST01
	LD	$10,3,XSET	;Y=$10(line 4)
YST01:	LD	$10,2		;Y=$10(line 3)
	AD	$9,6
;
;	set X address
;
XSET:	BIU	$6		;
	BIU	$6		;
	LD	$7,$6		;X=$9+$6x12
	BIU	$7		;
	AD	$7,$6		;
	AD	$9,$7		;
	SBC	$9,96		;check LCD block
	JR	C,DRW01		;
	AD	$9,32		;add offset for block2
DRW01:	CAL	LCDST		;drawing
	STLM	$15,5		;
	RTN
;
;	for G#2
;
GS2DR:	LD	$10,2		;drawing for G#2 key
	LDW	$8,&HDA82	;LCD1
	CAL	LCDST
	STLM	$15,3
	LDW	$8,&H0092	;LCD2
	CAL	LCDST
	STLW	$18
	RTN
;
;	set LCD command
;
LCDST:	PPO	&HC5
	STLM	$8,3
	PPO	&HC4
	RTN
;
;	draw keyboard (harf)
;
DRWKB:	CAL	LCDST
	LD	$1,64
DRKB1:	LDI	$0,(IZ+$31)
	LDM	$2,(IX+$0),6
	STLM	$2,6
	SB	$1,$30
	JR	NZ,DRKB1
	RTN
;
;	set I/O address
;
SETIO:	PRE	IX,&H8000
	PST	UA,&H75
	RTN
;
;	calc F parameter
;
CLFPR:	LD	$12,$31
CLF01:	SBC	$0,14
	JR	C,CLF02
	SB	$0,14
	AD	$12,$30,CLF01
CLF02:	AD	$12,$21
	JR	Z,CLF03
	PRE	IX,TNDT1
	SB	$12,$30
	BIU	$12
	OR	$12,$30,CLF04
CLF03:	PRE	IX,TNDT0
CLF04:	LDD	$11,(IX+$0)
	OR	$12,&H30
	RTN
;
;	key matrix data
;
KYDTM:	DB	&H7,&H80,&H6,&H80,&H6,&H40		;  BS     ^     CLS
	DB	&H8,&H80,&H7,&H40,&H8,&H40,&H9,&H80	;  *      /      -      +

KYDAT:	DB	&H8,&H80,&H7,&H80,&H9,&H80,&H6,&H40	; CAPS    A      Z      S
	DB	&H8,&H40,&H1,&H01,&H9,&H40,&H6,&H20	;  X             C      F
	DB	&H8,&H20,&H7,&H20,&H9,&H20,&H1,&H80	;  V      G      B     BRK
	DB	&H8,&H10,&H7,&H10,&H9,&H10,&H6,&H08	;  N      J      M      K
	DB	&H8,&H08,&H7,&H08,&H9,&H08,&H1,&H01	;  ,      L      =
	DB	&H8,&H04,&H6,&H04,&H9,&H04,&H7,&H04	; SPC    ANS     0      1
	DB	&H9,&H02,&H1,&H01,&H9,&H01,&H8,&H01	;  .             |E     3
	DB	&H4,&H80,&H3,&H80,&H5,&H80,&H2,&H40	;  Q     MEMO    W      IN
	DB	&H4,&H40,&H1,&H01,&H5,&H40,&H2,&H20	;  E             R     CALC
	DB	&H4,&H20,&H3,&H20,&H5,&H20,&H1,&H01	;  T           Y
	DB	&H4,&H10,&H3,&H10,&H5,&H10,&H2,&H08	;  U           I      
	DB	&H4,&H08,&H3,&H08,&H5,&H08,&H1,&H01	;  O     INS     P
	DB	&H4,&H04,&H2,&H04,&H5,&H04,&H3,&H04	; ENG   SHIFT    4      7
	DB	&H7,&H02,&H1,&H01,&H7,&H01,&H6,&H01	;  5             6      9
	DB	&H4,&H02,&H3,&H02,&H5,&H02,&H3,&H01	;  MR    log     M+     ln
;
;	tone data
;
TNDT0:	DB	&H81,&H89,&H91,&H9A,&HA3,&H00		; G0  G#0  A0  A#0  B0  --
	DB	&HAC,&HB7,&HC2,&HCD,&HD9,&H00		; C1  C#1  D1  D#1  E1  --
	DB	&HE6,&HF4				; F1  F#1

TNDT1:	DB	&H02,&H12,&H22,&H33,&H46,&H00		; G1  G#1  A1  A#1  B1  --
	DB	&H59,&H6D,&H83,&H9A,&HB3,&H00		; C2  C#2  D2  D#2  E2  --
	DB	&HCC,&HE8				; F2  F#2
;
;	keyboard graphic order data (LCD1)
;
ORDER:	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00 ; line1
	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00
	DB	&H0C,&H0C,&H0C,&H12,&H06,&H0C,&H12,&H06 ; line2
	DB	&H0C,&H0C,&H12,&H06,&H0C,&H12,&H06,&H0C
	DB	&H2A,&H1E,&H1E,&H24,&H18,&H1E,&H24,&H18 ; line3
	DB	&H1E,&H1E,&H24,&H18,&H1E,&H24,&H18,&H1E
	DB	&H30,&H36,&H36,&H36,&H36,&H36,&H36,&H36 ; line4
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36
;
;	keyboard graphic order data (LCD2)
;
	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00 ; line1
	DB	&H00,&H00,&H00,&H00,&H00,&H00,&H00,&H00
	DB	&H0C,&H12,&H06,&H0C,&H12,&H06,&H0C,&H0C	; line2
#if MODEL=FX870P
	DB	&H12,&H06,&H0C,&H12,&H06,&H0C,&H0C,&H0C
	DB	&H1E,&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E ; line3
	DB	&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E,&H3C
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36	; line4
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H42
#else
	DB	&H12,&H06,&H0C,&H12,&H06,&H0C,&H0C,&H12
	DB	&H1E,&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E ; line3
	DB	&H24,&H18,&H1E,&H24,&H18,&H1E,&H1E,&H24
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36	; line4
	DB	&H36,&H36,&H36,&H36,&H36,&H36,&H36,&H36
#endif
;
;	graphic data
;
CGDAT:	DB	&HFF,&HFF,&HFF,&HFF,&HFF,&HFF	;  0 (&H00)
	DB	&H00,&H00,&H00,&H00,&HFF,&HFF	;  1 (&H06)
	DB	&HFF,&H00,&H00,&H00,&HFF,&HFF	;  2 (&H0C)
	DB	&HFF,&H00,&H00,&H00,&H00,&HFF	;  3 (&H12)
	DB	&H00,&H00,&H00			;  4 (&H18)
BLACK:	DB	&H00,&HFC,&HFF			;  4
	DB	&HFC,&H00,&H00,&H00,&HFC,&HFF	;  5 (&H1E)
	DB	&HFC,&H00,&H00,&H00,&H00,&HFF	;  6 (&H24)
	DB	&HFF,&H00,&H00,&H00,&HFC,&HFF	;  7 (&H2A)
	DB	&HFE,&H01,&H01,&H01,&H01,&HFE	;  8 (&H30)
WHITE:	DB	&H01,&H01,&H01,&H01,&H01,&HFE	;  9 (&H36)
#if MODEL=FX870P
	DB	&HFC,&H00,&H00,&H00,&HFF,&H00	; 10 (&H3C)
	DB	&H01,&H01,&H01,&H01,&HFE,&H00	; 11 (&H42)
#endif
;
;	original tone data
;
ORGDT:	DB	&H21,&H21,&H30,&H30,&HFF,&HFF,&H00,&H00
;
;	input key buffer
;
KYBUF:	DB	&HFF,&HFF,&HFF,&HFF,&HFF,&HFF,&HFF,&HFF,&HFF
